/*
 * Written by Professor Bill Tomlinson for Winter 2004 Biomorphic Computing class at UCI.
 * wmt@uci.edu
 */
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

public class GridWorld {
	//Milliseconds to sleep
	public final int SLEEP_TIME = 20;
	//how many blocks in x
	public static int xSize = 50;
	//how many blocks in y
	public static int ySize = 50;
	//how many pixels per block
	public static int cellSize = 10;
	//in this version of the grid world:
	//0 means uninhabited
	//1 means the target is there
	//2 means the creature is there
	public static int[][] blocks = new int[xSize][ySize];
	Color backgroundColor = Color.gray;
	static Canvas gridCanvas;
	private static GridWorld thisGridWorld;
		public static Creature thisCreature;


	public GridWorld(GAMain ga, Creature c) {
		setCreature(c);
		ga.setLayout(new BorderLayout());
		//Panel p = new Panel();
		gridCanvas = new Canvas() {
			public void paint(Graphics g) {
				setBackground(backgroundColor);
				g.clearRect(
					1,
					1,
					(xSize * cellSize) - 2,
					(ySize * cellSize) - 2);

				for (int thisX = 0; thisX < xSize; thisX++) {
					for (int thisY = 0; thisY < ySize; thisY++) {

						if (blocks[thisX][thisY] == 1) {
							//System.out.println("Printing the target: "+thisX+", "+thisY);
							g.setColor(Color.black);
							g.fillRect(
								thisX * cellSize,
								thisY * cellSize,
								cellSize - 1,
								cellSize - 1);
						} else if (blocks[thisX][thisY] == 2) {
							//System.out.println("Printing the evolved creature: "+thisX+", "+thisY);
							g.setColor(Color.black);
							g.fillRect(
								thisX * cellSize,
								thisY * cellSize,
								cellSize - 1,
								cellSize - 1);
							g.setColor(Color.white);
							g.drawLine(
								thisX * cellSize + cellSize / 2 - 1,
								thisY * cellSize + cellSize / 2 - 1,
								(int) (thisX * cellSize
									+ cellSize / 2
									- 1
									+ GridWorld.getCreature().getForwardVec().getX()
										* (cellSize - 1)),
								(int) (thisY * cellSize
									+ cellSize / 2
									- 1
									+ GridWorld.getCreature().getForwardVec().getY()
										* (cellSize - 1)));
						} else {
							g.setColor(Color.white);
							g.fillRect(
								thisX * cellSize,
								thisY * cellSize,
								cellSize - 1,
								cellSize - 1);
						}
					}
				}
			}
		};
		Panel controlPanel = new Panel();
		controlPanel.setSize(30, 10);
		gridCanvas.setSize(500, 500);
		ga.setLayout(new BorderLayout());

		ga.add(BorderLayout.CENTER, gridCanvas);
		ga.add(BorderLayout.SOUTH, controlPanel);
		gridCanvas.addMouseListener(new GridWorldMouseListener());
		ga.setVisible(true);
		ga.validate();
		//setGridCanvas(gridCanvas);
		setGridWorld(this);
	}
	int creatureXPos, creatureYPos, lastCreatureXPos, lastCreatureYPos;
	Vec2 pos = new Vec2();
	public void runWorld()
	{
		while (true) {
			System.out.println("\nGridWorld.runWorld");
			thisCreature.update();
			pos = thisCreature.getPosition();
			lastCreatureXPos = creatureXPos;
			lastCreatureYPos = creatureYPos;
			creatureXPos = (int) (pos.getX() / GridWorld.cellSize);
			creatureYPos = (int) (pos.getY() / GridWorld.cellSize);
			if (lastCreatureXPos > 0
				&& lastCreatureXPos < GridWorld.xSize
				&& lastCreatureYPos > 0
				&& lastCreatureYPos < GridWorld.ySize) {
				GridWorld.blocks[lastCreatureXPos][lastCreatureYPos] = 0;
			}
			if (creatureXPos > 0
				&& creatureXPos < GridWorld.xSize
				&& creatureYPos > 0
				&& creatureYPos < GridWorld.ySize) {
				GridWorld.blocks[creatureXPos][creatureYPos] = 2;
			}
			gridCanvas.repaint();
			try {
				Thread.sleep(SLEEP_TIME);
			} catch (Exception e) {
			}
		}
	}
	public static GridWorld getGridWorld() {
		return thisGridWorld;
	}
	public static void setGridWorld(GridWorld gw) {
		thisGridWorld = gw;
	}
	public static Canvas getGridCanvas() {
		return gridCanvas;
	}
	public static Creature getCreature() {
		return thisCreature;
	}
	public static void setCreature(Creature c) {
		thisCreature = c;
	}
	public static void setClickedTarget(int xPos, int yPos) {
		clickedX = xPos;
		clickedY = yPos;
	}
	public static Vec2 getTarget() {
		return new Vec2(clickedX, clickedY);
	}

	static int clickedX = 0;
	static int clickedY = 0;

}
class GridWorldMouseListener implements MouseListener {
	int xPos, yPos, lastXPos, lastYPos;
	//this happens when you click, or at the beginning of a drag
	//whenever this happens it should reverse the block
	public void mousePressed(MouseEvent e) {
		lastXPos = xPos;
		lastYPos = yPos;
		xPos = e.getX() / GridWorld.cellSize;
		yPos = e.getY() / GridWorld.cellSize;
		GridWorld.blocks[lastXPos][lastYPos] = 0;
		GridWorld.blocks[xPos][yPos] = 1;
		GridWorld.setClickedTarget(xPos, yPos);
		GridWorld.gridCanvas.repaint();
	}
	public void mouseMoved(MouseEvent e) {
	}
	public void mouseReleased(MouseEvent e) {
	}
	public void mouseEntered(MouseEvent e) {
	}
	public void mouseExited(MouseEvent e) {
	}
	public void mouseClicked(MouseEvent e) {
	}
}
